package eu.etaxonomy.taxeditor.editor.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

public class OpenDerivateViewHandler extends AbstractHandler {

    /* (non-Javadoc)
     * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ISelection currentSelection = HandlerUtil.getCurrentSelection(event);
        if(currentSelection instanceof IStructuredSelection){
            Iterator<?> selectionIterator = ((IStructuredSelection) currentSelection).iterator();
            Set<UUID> selectedElementUUIDs = new HashSet<UUID>();
            while(selectionIterator.hasNext()){
                Object object = selectionIterator.next();
                if(object instanceof SpecimenOrObservationBase<?>){
                    selectedElementUUIDs.add(((SpecimenOrObservationBase<?>) object).getUuid());
                }
            }
            DerivateViewEditorInput input = new DerivateViewEditorInput(selectedElementUUIDs);
            try {
                EditorUtil.open(input);
            } catch (PartInitException e) {
                MessagingUtils.error(OpenDerivateViewHandler.class, "Could not open Derivate Editor", e);
            } catch (NullPointerException npe){
                MessagingUtils.messageDialog("Failed to open Editor", OpenDerivateViewHandler.class, "Could not open Derivate Editor. The derivate hierarchy is corrupted!", npe);
            }
        }
        return null;
    }

}
