/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.name.container;

import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * <p>AcceptedNameContainer class.</p>
 *
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class AcceptedNameContainer extends AbstractGroupedContainer<Taxon> {
	
	private AcceptedGroup acceptedGroup;


	/**
	 * <p>Constructor for AcceptedNameContainer.</p>
	 *
	 * @param editor a {@link eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor} object.
	 * @param group a {@link eu.etaxonomy.taxeditor.editor.name.container.AbstractGroup} object.
	 */
	public AcceptedNameContainer(Taxon taxon) {
		super(taxon);
	}


	/**
	 * <p>initializeComposite</p>
	 */
	public void initializeComposite(){
		setFont(getViewerFont());
		
		initTextViewer();
	}
	
	/** {@inheritDoc} */
	@Override
	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.ACCEPTED_TAXON_FONT);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer#setGroup(eu.etaxonomy.taxeditor.editor.name.AbstractGroup)
	 */
	public void setGroup(AcceptedGroup group) {
		this.acceptedGroup = group;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer#getGroup()
	 */
	@Override
	public AbstractGroup getGroup() {
		return acceptedGroup;
	}
	
	/** {@inheritDoc} */
	@Override
	protected void updateNonEditableInfo() {
		// not needed
	}
	

	@Override
	protected void updateIcon() {
		setIcon(ACCEPTED_ICON);		
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer#updateIndent()
	 */
	@Override
	protected void updateIndent() {
		setIndent(ACCEPTED_INDENT);
	}

}
