// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Wizard for querying specimen provider.
 * @author pplitzner
 * @date 11.09.2013
 *
 */
public class SpecimenSearchWizard extends Wizard implements IImportWizard {
    private static final Logger logger = Logger.getLogger(SpecimenSearchWizard.class);

    private OccurenceQuery query = null;
    private SpecimenProviderSelectionWizardPage providerSelectionPage;
    private SpecimenSearchWizardPage searchPage;

    /**
     * Creates a new SpecimenSearchWizard
     */
    public SpecimenSearchWizard() {
        //default constructor needed for RCP extension points

        //check if connected to a data source. If not this will open an error dialog
        CdmStore.getCurrentApplicationConfiguration();
        this.setWindowTitle("Search Specimens");
    }


    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.Wizard#performFinish()
     */
    @Override
    public boolean performFinish() {
        searchPage.getController().saveLastSate();
        providerSelectionPage.getController().saveLastState();

        DataImportView<?> dataImportView = null;
        query = searchPage.getQuery();
        try {
            switch (providerSelectionPage.getQueryType()) {
            case BIOCASE:
                SpecimenImportView specimenImportView = (SpecimenImportView) TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage()
                        .showView(SpecimenImportView.ID, "query_"+query.toString().replaceAll(",", ""),IWorkbenchPage.VIEW_ACTIVATE);
                specimenImportView.setEndpoint(providerSelectionPage.getEndPoint());
                dataImportView = specimenImportView;
                break;
            case GBIF:
                dataImportView = (GbifResponseImportView) TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage()
                .showView(GbifResponseImportView.ID, "query_"+query.toString().replaceAll(",", ""),IWorkbenchPage.VIEW_ACTIVATE);
                break;
            }
        } catch ( PartInitException e ) {
            logger.error("Could not open import view for query: " + query.toString(), e);
        }
        if(dataImportView!=null){
            dataImportView.setQuery(query);
//            dataImportView.query();
//            Display.getDefault().asyncExec(new Runnable() {
//
//                @Override
//                public void run() {
//                    CdmStore.getContextManager().notifyContextRefresh();
//                }
//            });
            Job queryJob = new QueryJob("Query specimen provider", dataImportView);
            queryJob.schedule();
        }
        return true;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.Wizard#addPages()
     */
    @Override
    public void addPages() {
        addPage(providerSelectionPage);
        addPage(searchPage);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
     */
    @Override
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        providerSelectionPage = new SpecimenProviderSelectionWizardPage("Select specimen provider");
        searchPage = new SpecimenSearchWizardPage("Specimen Search");
    }

    private class QueryJob extends Job{

        private final DataImportView<?> view;

        /**
         * @param name
         */
        public QueryJob(String name, DataImportView<?> view) {
            super(name);
            this.view = view;
        }

        /* (non-Javadoc)
         * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
         */
        @Override
        protected IStatus run(IProgressMonitor monitor) {
            view.getConversationHolder().bind();
            view.query();
            Display.getDefault().asyncExec(new Runnable() {

                @Override
                public void run() {
                    CdmStore.getContextManager().notifyContextRefresh();
                }
            });
            return Status.OK_STATUS;
        }

    }
}
