// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport.transientServices;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.hibernate.LockOptions;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.envers.query.criteria.AuditCriterion;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.pager.Pager;
import eu.etaxonomy.cdm.common.monitor.IProgressMonitor;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.ISourceable;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.LSID;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.view.AuditEvent;
import eu.etaxonomy.cdm.model.view.AuditEventRecord;
import eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort;
import eu.etaxonomy.cdm.persistence.query.Grouping;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.cdm.persistence.query.OrderHint;
import eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.merge.IMergeStrategy;

/**
 * @author pplitzner
 * @date 27.09.2013
 *
 */
public class TransientReferenceService implements IReferenceService {

    private IReferenceService defaultReferenceService;

    /**
     * @param defaultReferenceService
     */
    public TransientReferenceService(IReferenceService defaultReferenceService) {
        this.defaultReferenceService = defaultReferenceService;
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IReferenceService#search(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Reference> search(Class<? extends Reference> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.search(clazz, queryString, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(eu.etaxonomy.cdm.model.common.VersionableEntity, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<Reference>> pageAuditEvents(Reference t, Integer pageSize, Integer pageNumber, AuditEventSort sort, List<String> propertyPaths) {
        return defaultReferenceService.pageAuditEvents(t, pageSize, pageNumber, sort, propertyPaths);
    }

    /**
     * @param annotatedObj
     * @param status
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getAnnotations(eu.etaxonomy.cdm.model.common.AnnotatableEntity, eu.etaxonomy.cdm.model.common.MarkerType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Annotation> getAnnotations(Reference annotatedObj, MarkerType status, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.getAnnotations(annotatedObj, status, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IService#clear()
     */
    @Override
    public void clear() {
        defaultReferenceService.clear();
    }

    /**
     * @param t
     * @param lockMode
     * @see eu.etaxonomy.cdm.api.service.IService#lock(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode)
     */
    @Override
    public void lock(Reference t, LockOptions lockOptions) {
        defaultReferenceService.lock(t, lockOptions);
    }

    /**
     * @param t
     * @param lockMode
     * @param propertyPaths
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode, java.util.List)
     */
    @Override
    public void refresh(Reference t, LockOptions lockOptions, List<String> propertyPaths) {
        defaultReferenceService.refresh(t, lockOptions, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache()
     */
    @Override
    public void updateTitleCache() {
        defaultReferenceService.updateTitleCache();
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getNextAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<Reference> getNextAuditEvent(Reference t) {
        return defaultReferenceService.getNextAuditEvent(t);
    }

    /**
     * @param annotatableEntity
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getMarkers(eu.etaxonomy.cdm.model.common.AnnotatableEntity, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Marker> getMarkers(Reference annotatableEntity, Boolean technical, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.getMarkers(annotatableEntity, technical, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IReferenceService#getUuidAndTitle()
     */
    @Override
    public List<UuidAndTitleCache<Reference>> getUuidAndTitle() {
        return defaultReferenceService.getUuidAndTitle();
    }

    /**
     * @param clazz
     * @param stepSize
     * @param cacheStrategy
     * @param monitor
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache(java.lang.Class, java.lang.Integer, eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy, eu.etaxonomy.cdm.common.monitor.IProgressMonitor)
     */
    @Override
    public void updateTitleCache(Class<? extends Reference> clazz, Integer stepSize, IIdentifiableEntityCacheStrategy<Reference> cacheStrategy, IProgressMonitor monitor) {
        defaultReferenceService.updateTitleCache(clazz, stepSize, cacheStrategy, monitor);
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getPreviousAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<Reference> getPreviousAuditEvent(Reference t) {
        return defaultReferenceService.getPreviousAuditEvent(t);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IReferenceService#getAllReferencesForPublishing()
     */
    @Override
    public List<Reference> getAllReferencesForPublishing() {
        return defaultReferenceService.getAllReferencesForPublishing();
    }

    /**
     * @param clazz
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#count(java.lang.Class)
     */
    @Override
    public int count(Class<? extends Reference> clazz) {
        return defaultReferenceService.count(clazz);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IReferenceService#getAllNomenclaturalReferences()
     */
    @Override
    public List<Reference> getAllNomenclaturalReferences() {
        return defaultReferenceService.getAllNomenclaturalReferences();
    }

    /**
     * @param reference
     * @param includeSubordinateReferences
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IReferenceService#listCoveredTaxa(eu.etaxonomy.cdm.model.reference.Reference, boolean, java.util.List)
     */
    @Override
    public List<TaxonBase> listCoveredTaxa(Reference reference, boolean includeSubordinateReferences, List<String> propertyPaths) {
        return defaultReferenceService.listCoveredTaxa(reference, includeSubordinateReferences, propertyPaths);
    }

    /**
     * @param clazz
     * @param from
     * @param to
     * @param criteria
     * @param pageSize
     * @param pageValue
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(java.lang.Class, eu.etaxonomy.cdm.model.view.AuditEvent, eu.etaxonomy.cdm.model.view.AuditEvent, java.util.List, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<Reference>> pageAuditEvents(Class<? extends Reference> clazz, AuditEvent from, AuditEvent to, List<AuditCriterion> criteria, Integer pageSize, Integer pageValue, AuditEventSort sort, List<String> propertyPaths) {
        return defaultReferenceService.pageAuditEvents(clazz, from, to, criteria, pageSize, pageValue, sort, propertyPaths);
    }

    /**
     * @param lsid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#find(eu.etaxonomy.cdm.model.common.LSID)
     */
    @Override
    public Reference find(LSID lsid) {
        return defaultReferenceService.find(lsid);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#delete(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public DeleteResult delete(Reference persistentObject) {
        return defaultReferenceService.delete(persistentObject);
    }

    /**
     * @param clazz
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#groupMarkers(java.lang.Class, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<Object[]> groupMarkers(Class<? extends Reference> clazz, Boolean technical, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultReferenceService.groupMarkers(clazz, technical, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#exists(java.util.UUID)
     */
    @Override
    public boolean exists(UUID uuid) {
        return defaultReferenceService.exists(uuid);
    }

    /**
     * @param x
     * @param y
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#replace(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
     */
    @Override
    public Reference replace(Reference x, Reference y) {
        return defaultReferenceService.replace(x, y);
    }

    /**
     * @param uuidSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.Set)
     */
    @Override
    public List<Reference> find(Set<UUID> uuidSet) {
        return defaultReferenceService.find(uuidSet);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSources(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<IdentifiableSource> getSources(Reference t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultReferenceService.getSources(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.UUID)
     */
    @Override
    public Reference find(UUID uuid) {
        return defaultReferenceService.find(uuid);
    }

    /**
     * @param id
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(int)
     */
    @Override
    public Reference find(int id) {
        return defaultReferenceService.find(id);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getRights(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<Rights> getRights(Reference t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultReferenceService.getRights(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param clazz
     * @param technical
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#countMarkers(java.lang.Class, java.lang.Boolean)
     */
    @Override
    public int countMarkers(Class<? extends Reference> clazz, Boolean technical) {
        return defaultReferenceService.countMarkers(clazz, technical);
    }

    /**
     * @param idSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#findById(java.util.Set)
     */
    @Override
    public List<Reference> findById(Set<Integer> idSet) {
        return defaultReferenceService.findById(idSet);
    }

    /**
     * @param clazz
     * @param idInSource
     * @param idNamespace
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSourcedObjectByIdInSource(java.lang.Class, java.lang.String, java.lang.String)
     */
    @Override
    public ISourceable getSourcedObjectByIdInSource(Class clazz, String idInSource, String idNamespace) {
        return defaultReferenceService.getSourcedObjectByIdInSource(clazz, idInSource, idNamespace);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#getSession()
     */
    @Override
    public Session getSession() {
        return defaultReferenceService.getSession();
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<Reference>> getUuidAndTitleCache() {
        return defaultReferenceService.getUuidAndTitleCache();
    }

    /**
     * @param clazz
     * @param limit
     * @param start
     * @param groups
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#group(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Object[]> group(Class<? extends Reference> clazz, Integer limit, Integer start, List<Grouping> groups, List<String> propertyPaths) {
        return defaultReferenceService.group(clazz, limit, start, groups, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Reference> findByTitle(Class<? extends Reference> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.findByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param type
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public  <S extends Reference> List<S> list(Class<S> type, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.list(type, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Pager<Reference> findByTitle(IIdentifiableEntityServiceConfigurator<Reference> configurator) {
        return defaultReferenceService.findByTitle(configurator);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public Integer countByTitle(Class<? extends Reference> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria) {
        return defaultReferenceService.countByTitle(clazz, queryString, matchmode, criteria);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID)
     */
    @Override
    public Reference load(UUID uuid) {
        return defaultReferenceService.load(uuid);
    }

    /**
     * @param uuid
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID, java.util.List)
     */
    @Override
    public Reference load(UUID uuid, List<String> propertyPaths) {
        return defaultReferenceService.load(uuid, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Integer countByTitle(IIdentifiableEntityServiceConfigurator<Reference> configurator) {
        return defaultReferenceService.countByTitle(configurator);
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#merge(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public Reference merge(Reference transientObject) {
        return defaultReferenceService.merge(transientObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Reference> listByTitle(Class<? extends Reference> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.listByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#page(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public  <S extends Reference> Pager<S> page(Class<S> type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.page(type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID refresh(Reference persistentObject) {
        return defaultReferenceService.refresh(persistentObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByReferenceTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Reference> listByReferenceTitle(Class<? extends Reference> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.listByReferenceTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param tableName
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#rows(java.lang.String, int, int)
     */
    @Override
    public List<Reference> rows(String tableName, int limit, int start) {
        return defaultReferenceService.rows(tableName, limit, start);
    }

    /**
     * @param newInstances
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(java.util.Collection)
     */
    @Override
    public Map<UUID, Reference> save(Collection<Reference> newInstances) {
        return null;
    }

    /**
     * @param newInstance
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID save(Reference newInstance) {
        return null;
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID saveOrUpdate(Reference transientObject) {
        return null;
    }

    /**
     * @param transientObjects
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(java.util.Collection)
     */
    @Override
    public Map<UUID, Reference> saveOrUpdate(Collection<Reference> transientObjects) {
        return null;
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#update(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID update(Reference transientObject) {
        return defaultReferenceService.update(transientObject);
    }

    /**
     * @param example
     * @param includeProperties
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(eu.etaxonomy.cdm.model.common.ICdmBase, java.util.Set, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Reference> list(Reference example, Set<String> includeProperties, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultReferenceService.list(example, includeProperties, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param matchStrategy
     * @param mergeStrategy
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#deduplicate(java.lang.Class, eu.etaxonomy.cdm.strategy.match.IMatchStrategy, eu.etaxonomy.cdm.strategy.merge.IMergeStrategy)
     */
    @Override
    public int deduplicate(Class<? extends Reference> clazz, IMatchStrategy matchStrategy, IMergeStrategy mergeStrategy) {
        return defaultReferenceService.deduplicate(clazz, matchStrategy, mergeStrategy);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param matchMode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findTitleCache(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, eu.etaxonomy.cdm.persistence.query.MatchMode)
     */
    @Override
    public Pager<Reference> findTitleCache(Class<? extends Reference> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, MatchMode matchMode) {
        return defaultReferenceService.findTitleCache(clazz, queryString, pageSize, pageNumber, orderHints, matchMode);
    }

	@Override
	public DeleteResult isDeletable(Reference object,
			DeleteConfiguratorBase config) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Reference findWithoutFlush(UUID uuid) {
		return defaultReferenceService.findWithoutFlush(uuid);
	}


}
