package eu.etaxonomy.taxeditor.editor.view.derivate.contextMenu;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeCacheStrategy;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivationEventType;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * Context menu for the creation of derivates in the derivate hierarchy.
 *
 */
public class CreateDerivateContextMenu extends CompoundContributionItem {

    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
     */
    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        MenuItem addItem = new MenuItem(menu, SWT.CASCADE);
                        addItem.setText("Add...");
                        Menu addMenu = new Menu(menu);
                        addItem.setMenu(addMenu);
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        final ISelection selection = window.getActivePage().getSelection();
                        TreeNode selectedTreeNode = EditorUtil.getTreeNodeOfSelection(selection);
                        if(selectedTreeNode!=null){
                            //context menu for FieldUnit
                            if(selectedTreeNode.getValue() instanceof FieldUnit){
                                createMenuItem(addMenu, "Specimen", selectedTreeNode, DerivedUnit.class, SpecimenOrObservationType.PreservedSpecimen, DerivationEventType.GATHERING_IN_SITU());
                                new MenuItem(addMenu, SWT.SEPARATOR);
                                createMenuItem(addMenu, "Tissue Sample", selectedTreeNode, DnaSample.class, SpecimenOrObservationType.TissueSample, DerivationEventType.PREPARATION());
                                createMenuItem(addMenu, "DNA Sample", selectedTreeNode, DnaSample.class, SpecimenOrObservationType.DnaSample, DerivationEventType.DNA_EXTRACTION());
                                createMenuItem(addMenu, "Media Specimen", selectedTreeNode, MediaSpecimen.class, SpecimenOrObservationType.Media, DerivationEventType.ACCESSIONING());
                            }
                            //context menu for Specimen
                            else if(selectedTreeNode.getValue() instanceof DerivedUnit && ((DerivedUnit)selectedTreeNode.getValue()).getRecordBasis()==SpecimenOrObservationType.PreservedSpecimen){

                                createMenuItem(addMenu, "Tissue Sample", selectedTreeNode, DnaSample.class, SpecimenOrObservationType.TissueSample, DerivationEventType.PREPARATION());
                                createMenuItem(addMenu, "DNA Sample", selectedTreeNode, DnaSample.class, SpecimenOrObservationType.DnaSample, DerivationEventType.DNA_EXTRACTION());
                                createMenuItem(addMenu, "Media Specimen", selectedTreeNode, MediaSpecimen.class, SpecimenOrObservationType.Media, DerivationEventType.ACCESSIONING());
                            }
                            //context menu for TissueSample
                            else if(selectedTreeNode.getValue() instanceof DnaSample && ((DnaSample)selectedTreeNode.getValue()).getRecordBasis()==SpecimenOrObservationType.TissueSample){
                            	createMenuItem(addMenu, "DnaSample", selectedTreeNode, DnaSample.class, SpecimenOrObservationType.DnaSample, DerivationEventType.DNA_EXTRACTION());
                            	createMenuItem(addMenu, "Media Specimen", selectedTreeNode, MediaSpecimen.class, SpecimenOrObservationType.Media, DerivationEventType.ACCESSIONING());
                            }
                            //context menu for DnaSample
                            else if(selectedTreeNode.getValue() instanceof DnaSample && ((DnaSample)selectedTreeNode.getValue()).getRecordBasis()==SpecimenOrObservationType.DnaSample){
                                createMenuItem(addMenu, "Consensus Sequence", selectedTreeNode, Sequence.class, null, null);
                                createMenuItem(addMenu, "Media Specimen", selectedTreeNode, MediaSpecimen.class, SpecimenOrObservationType.Media, DerivationEventType.ACCESSIONING());
                            }
                            //context menu for Sequence
                            else if(selectedTreeNode.getValue() instanceof Sequence){
                                createMenuItem(addMenu, "Single Read", selectedTreeNode, SingleRead.class, null, null);
                            }
                            //default
                            else{
                                MenuItem menuItem = new MenuItem(addMenu, SWT.NONE);
                                menuItem.setText("No child derivates");
                                menuItem.setEnabled(false);
                            }
                        }
                    }


                }
        };
        return contributionItems;
    }

    private void createMenuItem(Menu menu, String menuLabel, TreeNode selectedNode, Class<?> childClass, SpecimenOrObservationType specimenType, DerivationEventType derivationEventType) {
        MenuItem menuItem = new MenuItem(menu, SWT.NONE);
        menuItem.setText(menuLabel);
        menuItem.addSelectionListener(new WidgetSelectionListener(selectedNode, childClass, specimenType, derivationEventType));
    }

    private class WidgetSelectionListener extends SelectionAdapter{

        private final TreeNode selectedNode;
        //TODO: we need a common interface for the generic like e.g. "IDerivateHierarchyNode"
        private final Class<?> childClass;
        private final SpecimenOrObservationType specimenType;
        private final DerivationEventType derivationEventType;

        /**
         * @param selectedNode
         * @param specimenType
         */
        public WidgetSelectionListener(TreeNode selectedNode, Class<?> childClass, SpecimenOrObservationType specimenType, DerivationEventType derivationEventType) {
            this.selectedNode = selectedNode;
            this.childClass = childClass;
            this.specimenType = specimenType;
            this.derivationEventType = derivationEventType;
        }

        /* (non-Javadoc)
         * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetSelected(SelectionEvent e) {
            DerivateView derivateView = null;
            if(AbstractUtility.getActivePart() instanceof DerivateView){
                derivateView = (DerivateView) AbstractUtility.getActivePart();
            }
            if(derivateView!=null){
                if(derivateView.isDirty()){
                    e.doit = false;
                    MessagingUtils.warningDialog("View has unsaved changes", this, "You need to save before performing this action");
                    return;
                }
            }

            Object createdElement = null;
            Object selectedDerivate = selectedNode.getValue();
            if(selectedDerivate instanceof DnaSample && ((DnaSample) selectedDerivate).getRecordBasis()==SpecimenOrObservationType.DnaSample){
                DnaSample dnaSample = (DnaSample)selectedDerivate;
                if(childClass.equals(Sequence.class)){
                    Sequence newInstance = Sequence.NewInstance(dnaSample, "", 0);
                    createdElement = newInstance;
                }
            }
            else if(selectedDerivate instanceof Sequence){
                SingleRead newInstance = SingleRead.NewInstance();
                ((Sequence) selectedDerivate).addSingleRead(newInstance);
                createdElement = newInstance;
            }
            else if(selectedDerivate instanceof SpecimenOrObservationBase && DerivedUnit.class.isAssignableFrom(childClass)){
                SpecimenOrObservationBase<?> specimenOrObservationBase = (SpecimenOrObservationBase<?>)selectedDerivate;
                DerivedUnit derivedUnit;
                if(childClass.equals(DnaSample.class)){
                    derivedUnit = DnaSample.NewInstance();
                    if(specimenType==SpecimenOrObservationType.DnaSample){
                        derivedUnit.setRecordBasis(SpecimenOrObservationType.DnaSample);
                    }
                    else if(specimenType==SpecimenOrObservationType.TissueSample){
                        derivedUnit.setRecordBasis(SpecimenOrObservationType.TissueSample);
                    }
                }
                else if(childClass.equals(MediaSpecimen.class)){
                    derivedUnit = MediaSpecimen.NewInstance(specimenType);
                    //a MediaSpecimen with no media attached does not make sense. Hence we add one
                    ((MediaSpecimen)derivedUnit).setMediaSpecimen(Media.NewInstance());
                }
                else{
                    derivedUnit = DerivedUnit.NewInstance(specimenType);
                }
                specimenOrObservationBase.addDerivationEvent(DerivationEvent.NewSimpleInstance(specimenOrObservationBase, derivedUnit, derivationEventType));
                derivedUnit.setCacheStrategy(new DerivedUnitFacadeCacheStrategy());
                derivedUnit.getTitleCache(); //update title cache
                createdElement = derivedUnit;
            }

            //refresh view
            if(derivateView!=null){
                derivateView.getConversationHolder().commit();
                derivateView.refreshTree(createdElement);
            }
        }
    }

}
