/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.uses.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;


/**
 * CreateUseSummaryOperation Class
 * @author a.theys
 * @created mar 13, 2012
 * @version 1.0
 */
public class CreateUseSummaryOperation extends AbstractPostOperation {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.use.createUseRecord";

	private final DescriptionBase<?> description;
	private final Feature feature;
	private DescriptionElementBase element;


	/**
	 * <p>Constructor for CreateUseSummaryOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.TaxonDescription} object.
	 * @param feature a {@link eu.etaxonomy.cdm.model.description.Feature} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateUseSummaryOperation(String label, IUndoContext undoContext, DescriptionBase<?> description, Feature feature, IPostOperationEnabled postOperationEnabled) {
			super(label, undoContext, (Taxon)null, postOperationEnabled);

			this.description = description;
			this.feature = feature;
	}

	/**
	 * <p>Constructor for CreateUseSummaryOperation.</p>
	 *
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.TaxonDescription} object.
	 * @param feature a {@link eu.etaxonomy.cdm.model.description.Feature} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param label a {@link java.lang.String} object.
	 */
	public CreateUseSummaryOperation(String label,
			IUndoContext undoContext, Taxon taxon,
			TaxonDescription description, Feature feature,
			DescriptionElementBase element, IPostOperationEnabled postOperationEnabled) {
			this(label, undoContext, description, feature, postOperationEnabled);

		this.element = element;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		monitor.worked(20);
		if (element == null) {

				element = TextData.NewInstance();
		}


		element.setFeature(feature);
		description.addElement(element);
		monitor.worked(40);

		return postExecute(element);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		description.addElement(element);

		return postExecute(element);
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		description.removeElement(element);

		return postExecute(null);
	}
}
